/* Copyright 2013 Andrey Dubovik <andrei.dubovik@gmail.com>
 *
 * This file is part of Minido.
 *
 * Minido is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Minido is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Minido. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef DATABASE_HPP
#define DATABASE_HPP
#include <vector>
#include <string>
#include <gtk/gtk.h>

class Item;
typedef std::vector<char> MatchMask;

class SearchResult {
public:
  Item* item;
  MatchMask mask;
  SearchResult();
  operator bool() const;
};

class DataBase {
private:
  std::vector<Item*> items;
  int size;
  int tick;

public:
  friend Item;
  DataBase();
  SearchResult search(const std::u32string& pattern);
};

class Icon {
private:
  GdkPixbuf* icon;

public:
  Icon();
  Icon(const std::string& name);
  Icon(const Icon&) = delete;
  Icon(Icon&& rhs);
  Icon& operator=(const Icon&) = delete;
  Icon& operator=(Icon&& rhs);
  ~Icon();
  operator GdkPixbuf*() const;
};

class Item {
private:
  int id;
  DataBase& db;
  std::u32string name;
  std::u32string lcname;
  std::u32string lcpatt;
  int tick;

public:
  friend DataBase;
  Icon icon;
  Item(DataBase& db);
  Item(const Item&) = delete;
  Item& operator=(const Item&) = delete;
  ~Item();
  void setName(const std::u32string& name);
  void setName(const std::u32string&& name);
  const std::u32string& getName() const;


  virtual void invoke();
  void invoke(const std::u32string& patt);
};

class LScore {
public:
  int alpha; // Connectedness
  int beta; // Earliness
  LScore();
  bool operator<(const LScore& rhs) const;
};

class GScore {
public:
  int alpha; // Connectedness
  int beta; // Earliness
  LScore similar;
  int tick;
  int length;
  GScore();
  GScore(const LScore& lscore);
  bool operator<(const GScore& rhs) const;
};

LScore isSimilar(const std::u32string& str1, const std::u32string& str2);

LScore computeScore(const std::u32string& name,
		   const std::u32string& pattern,
		   int nc = 0, int pc = 0, int alpha = 0,
		   int sublen = 0, int beta = 0);

LScore computeMask(const std::u32string& name,
		   const std::u32string& pattern,
		   MatchMask& mask,
		   int nc = 0, int pc = 0, int alpha = 0,
		   int sublen = 0, int beta = 0);


#endif

